from __future__ import annotations
from cnnecharts.optionCaller import OptionCaller
import pandas as pd
from typing import TYPE_CHECKING, Optional

from cnnecharts.spec import Spec

if TYPE_CHECKING:
    from cnnecharts.mapping import Mapping


class Title(OptionCaller):
    def __init__(self, text: str) -> None:
        super().__init__()
        self.text = text

    def _ex_create_spec(self, mapping: Mapping, spec: Spec):
        spec["title"] = {"text": self.text}
        return spec


class Toolbox(OptionCaller):
    """docstring for Bar."""

    def __init__(self, show=True) -> None:
        super().__init__()
        self.show = show

    def _ex_create_spec(self, mapping: Mapping, spec: Spec):
        spec["toolbox"] = {"show": self.show}
        return spec



from __future__ import annotations
from cnnecharts.optionCaller import OptionCaller
import pandas as pd
from typing import TYPE_CHECKING, Optional

from cnnecharts.spec import Spec

if TYPE_CHECKING:
    from cnnecharts.mapping import Mapping

from dataclasses import dataclass, field


@dataclass
class MappingData(object):
    data: Optional[pd.DataFrame] = field(init=False, default=None)
    x: Optional[str] = field(init=False, default=None)
    y: Optional[str] = field(init=False, default=None)
    color: Optional[str] = field(init=False, default=None)


class ChartPart(OptionCaller):
    def __init__(self, *, id: Optional[str] = None) -> None:
        super().__init__()
        self.series_callers = []
        self._mappingData = MappingData()

        if id:

            def set_fn(series_obj):
                if "name" in series_obj:
                    series_obj["id"] = f'{series_obj["name"]}_{id}'
                else:
                    series_obj["id"] = id
                return series_obj

            self.series_callers.append(set_fn)

    def mapping(
        self,
        *,
        data: Optional[pd.DataFrame] = None,
        x: Optional[str],
        y: Optional[str],
        color: Optional[str],
    ):
        self._mappingData.data = data
        self._mappingData.x = x
        self._mappingData.y = y
        self._mappingData.color = color

        return self


class Bar(ChartPart):
    def __init__(self, *, id: Optional[str] = None) -> None:
        super().__init__(id=id)

    def onUniversalTransition(self):
        """开启动画效果"""

        def set_fn(series_obj):
            series_obj["universalTransition"] = True
            series_obj["animationDurationUpdate"] = 1000
            return series_obj

        self.series_callers.append(set_fn)
        return self

    def background(self, color: str):
        def set_fn(series_obj):
            series_obj["showBackground"] = True
            series_obj["backgroundStyle.color"] = color
            return series_obj

        self.series_callers.append(set_fn)

        return self

    def emphasis(self, focus="self"):
        def set_fn(series_obj):
            series_obj["emphasis.focus"] = focus
            return series_obj

        self.series_callers.append(set_fn)

        return self

    def _ex_create_spec(self, mapping: Mapping, spec: Spec):
        data = self._mappingData.data
        x = self._mappingData.x
        y = self._mappingData.y
        color = self._mappingData.color

        data, x, y, color = mapping.transform(data, x, y, color)

        xAxis = spec["xAxis"]
        xAxis["type"] = "category"
        xAxis["data"] = data.index.tolist()

        spec["yAxis.type"] = "value"

        series = [
            {
                "type": "bar",
                "name": col,
                "data": list(data[col]),
            }
            for col in data.columns
        ]

        for series_obj in series:
            for caller in self.series_callers:
                series_obj = caller(series_obj)

            spec.series.append(series_obj)

        spec["legend.data"] = list(data.columns)

        return spec


class Line(ChartPart):
    def __init__(self, *, id: Optional[str] = None) -> None:
        super().__init__(id=id)

    def onUniversalTransition(self):
        """开启动画效果"""

        def set_fn(series_obj):
            series_obj["universalTransition"] = True
            series_obj["animationDurationUpdate"] = 1000
            return series_obj

        self.series_callers.append(set_fn)
        return self

    def background(self, color: str):
        def set_fn(series_obj):
            series_obj["showBackground"] = True
            series_obj["backgroundStyle.color"] = color
            return series_obj

        self.series_callers.append(set_fn)

        return self

    def emphasis(self, focus="self"):
        def set_fn(series_obj):
            series_obj["emphasis.focus"] = focus
            return series_obj

        self.series_callers.append(set_fn)

        return self

    def _ex_create_spec(self, mapping: Mapping, spec: Spec):
        data = self._mappingData.data
        x = self._mappingData.x
        y = self._mappingData.y
        color = self._mappingData.color

        data, x, y, color = mapping.transform(data, x, y, color)

        xAxis = spec["xAxis"]
        xAxis["type"] = "category"
        xAxis["data"] = data.index.tolist()

        spec["yAxis.type"] = "value"

        series = [
            {
                "type": "line",
                "name": col,
                "data": list(data[col]),
            }
            for col in data.columns
        ]

        for series_obj in series:
            for caller in self.series_callers:
                series_obj = caller(series_obj)

            spec.series.append(series_obj)

        spec["legend.data"] = list(data.columns)

        return spec



from typing import Dict, List, Any, Optional
import json


class Spec:
    def __init__(self, init_data: Optional[Dict] = None) -> None:
        # tree = lambda: defaultdict(tree)
        self._data = init_data or {}
        self["series"] = []

    @property
    def series(self) -> List[Any]:
        """The series property."""
        if "series" not in self._data:
            self["series"] = []

        return self["series"]

    def __getitem__(self, __key: str) -> Any:
        paths = __key.split(".")

        data = self._data
        for p in paths:

            if p not in data:
                data[p] = {}
            data = data[p]

        return data

    def __setitem__(self, key: str, value: Any):
        paths = key.split(".")

        data = self._data
        for p in paths[:-1]:
            if p not in data:
                data[p] = {}
            data = data[p]
        data[paths[-1]] = value

    def __delitem__(self, key: str):
        paths = key.split(".")

        data = self._data
        for p in paths[:-1]:
            if p not in data:
                data[p] = {}
            data = data[p]

        del data[paths[-1]]

    def __str__(self) -> str:
        return json.dumps(self._data, indent=2)

    def __repr__(self) -> str:
        return str(self)



from __future__ import annotations
from typing import TYPE_CHECKING, Optional, Dict
import pprint
from cnnecharts.spec import Spec
import cnnecharts.utils.renders as renders
from cnnecharts.utils.data_gen import json_dumps_fn

if TYPE_CHECKING:
    from cnnecharts.mapping import Mapping


class OptionCaller(object):
    def __init__(self) -> None:
        pass

    def __add__(self, other: "OptionCaller"):
        store = OptionCallerStore()

        OptionCaller._caller2store(store, self)
        OptionCaller._caller2store(store, other)

        return store

    @staticmethod
    def _caller2store(store: "OptionCallerStore", caller: "OptionCaller"):
        if isinstance(caller, OptionCallerStore):
            store.add_callers(*caller.callers)
            store.mapping = caller.mapping
        else:
            store.add_callers(caller)

    def _ex_create_spec(self, mapping: Mapping, spec: Spec):
        return spec


class OptionCallerStore(OptionCaller):
    def __init__(self) -> None:
        super().__init__()
        self.mapping: Optional[Mapping] = None
        self.callers: list[OptionCaller] = []

    def add_callers(self, *caller: OptionCaller):
        self.callers.extend(caller)
        return self

    def _ex_create_spec(self, mapping: Mapping, spec: Spec):
        for caller in self.callers:
            spec = caller._ex_create_spec(mapping, spec)

        return spec

    def _repr_html_(
        self,
    ):
        return pprint.pformat(self.create_option(), indent=2)

    def create_option(self, option: Optional[Dict] = None):
        if self.mapping is None:
            raise ValueError("没有设置 mapping")

        spec = Spec(option)

        self._ex_create_spec(self.mapping, spec)
        return spec

    def to_html(self, path: str):
        content = json_dumps_fn(self.create_option()._data)
        content = renders.htmlRender(content)

        assert isinstance(content, str)
        with open(path, mode="w", encoding="utf8") as f:
            f.write(content)



from __future__ import annotations
from cnnecharts.mapping import Mapping

from typing import TYPE_CHECKING, List

# if TYPE_CHECKING:
#     from easyChart.layer import Layer


# class ChartOptions(object):
#     def __init__(self, mapping: Mapping) -> None:
#         self.mapping = mapping
#         self.layer: List[Layer] = []

#     def add(self, layer: Layer):
#         self.layer.append(layer)
#         return self



import pandas as pd
from typing import Union, Optional, List

from cnnecharts.optionCaller import OptionCaller, OptionCallerStore


class Mapping(object):
    def __init__(
        self,
        *,
        data: Optional[pd.DataFrame] = None,
        x: Optional[str] = None,
        y: Optional[str] = None,
        color: Optional[Union[str, List[str]]] = None,
    ) -> None:
        self.data = data
        self.x = x
        self.y = y
        self.color = color

    def __add__(self, other: OptionCaller):
        if not isinstance(other, OptionCaller):
            raise TypeError("mapping 必须与 OptionCaller 相加")

        if not isinstance(other, OptionCallerStore):
            other = OptionCallerStore().add_callers(other)

        other.mapping = self
        return other

    def transform(
        self,
        data: Optional[pd.DataFrame] = None,
        x: Optional[str] = None,
        y: Optional[str] = None,
        color: Optional[Union[str, List[str]]] = None,
    ):
        data = data or self.data

        if data is None or (not isinstance(data, pd.DataFrame)):
            raise TypeError("data must be DataFrame")

        x = x or self.x
        y = y or self.y
        color = color or self.color

        if x is None:
            raise ValueError("x must be seted")

        if color is None and y is None:
            raise ValueError("y must be seted")

        def tran_data(data: pd.DataFrame, x, y, color):

            if isinstance(color, list):
                if y is not None:
                    raise ValueError("y must be none when color is list")
                return data.pivot_table(index=x, values=color, aggfunc="mean")

            return data.pivot_table(index=x, columns=color, values=y, aggfunc="mean")

        data = tran_data(data, x, y, color)

        return data, x, y, color
